<?php
/**
 * @file
 * All the functions to connect to the external service.
 */

define('WEBTEXTTOOL_URL', 'https://app.webtexttool.com/api');

/**
 * Get an account.
 */
function webtexttool_account() {
  // Get the current account.
  $account = _webtexttool_connect('/user/info', _webtexttool_get_token());
  if ($account) {
    foreach ($account as $key => $value) {
      // Change all keys to lower.
      $temp_key = strtolower($key);
      unset($account[$key]);
      $account[$temp_key] = $value;
    }
    return theme('webtexttool-account', $account);
  }
  return t('Unable to load your account. Please check and update your credentials at !url.', array('!url' => l(t('My account'), 'admin/config/services/webtexttool/settings')));
}

/**
 * Connect to the webtexttool server.
 */
function _webtexttool_connect($function, $token = '', $parameters = array(), $method = 'GET', $status_code = '200') {

  $url = WEBTEXTTOOL_URL . $function;

  $headers = array('Content-Type' => 'application/json', 'WttSource' => 'Drupal');
  if ($token != '') {
    $headers['Authorization'] = 'bearer ' . $token;
  }

  $is_authenticated_options = array(
    'method' => $method,
    'timeout' => 60,
    'headers' => $headers,
  );
  $is_authenticated = drupal_http_request($url . '/user/authenticated', $is_authenticated_options);

  if (!$is_authenticated
    || $is_authenticated->status_message != 'OK'
    || $is_authenticated->data != 'true') {
    $login = _webtexttool_login();
    if (!$login) {
      drupal_set_message(t('Unable to login with the given webtexttool credentials.'), 'error');
      return FALSE;
    }
  }

  $options = array(
    'method' => $method,
    'data' => drupal_json_encode($parameters),
    'timeout' => 60,
    'headers' => $headers,
  );

  $response = drupal_http_request($url, $options);

  // Check response code.
  if ($response->code != $status_code) {
    drupal_set_message(t('Unable to connect to the online webtexttool service.'), 'error');
    return FALSE;
  }
  // Check if response is set.
  if ($response && isset($response->data) &&  $response->data != 'false') {
    $response = drupal_json_decode($response->data);
  }
  return $response;
}

/**
 * Register at the webtexttool server.
 */
function webtexttool_register($email, $pass, $language) {
  $url = WEBTEXTTOOL_URL . '/user/register';
  $headers = array('Content-Type' => 'application/json', 'WttSource' => 'Drupal');

  $data_array = array(
    'Language' => $language,
    'RememberMe' => TRUE,
    'UserName' => $email,
    'Password' => $pass,
  );
  $options = array(
    'method' => 'POST',
    'data' => drupal_json_encode($data_array),
    'timeout' => 60,
    'headers' => $headers,
  );
  $response = drupal_http_request($url, $options);

  // Check response code.
  if ($response->code != '200') {
    return FALSE;
  }
  // Check if response data is present.
  if ($response && isset($response->data) &&  $response->data != 'false') {
    $response = drupal_json_decode($response->data);
    if (isset($response['access_token']) && $response['access_token'] != '') {
      _webtexttool_set_token($response['access_token']);
      return $response;
    }
  }
  return FALSE;
}

/**
 * Login at the webtexttool server.
 */
function _webtexttool_login() {
  $url = WEBTEXTTOOL_URL . '/user/Login';
  $headers = array('Content-Type' => 'application/json', 'WttSource' => 'Drupal');

  $data_array = array(
    'Language' => 'en',
    'RememberMe' => TRUE,
    'UserName' => variable_get('webtexttool_user', ''),
    'Password' => variable_get('webtexttool_pass', ''),
  );
  $options = array(
    'method' => 'POST',
    'data' => drupal_json_encode($data_array),
    'timeout' => 60,
    'headers' => $headers,
  );
  $response = drupal_http_request($url, $options);

  // Check response code.
  if ($response->code != '200') {
    return FALSE;
  }
  // Check if response data is present.
  if ($response && isset($response->data) &&  $response->data != 'false') {
    $response = drupal_json_decode($response->data);
    if (isset($response['access_token']) && $response['access_token'] != '') {
      _webtexttool_set_token($response['access_token']);
      return $response;
    }
  }
  return FALSE;
}

/**
 * Helper function to check if user is authenticated.
 */
function _webtexttool_is_authenticated() {
  $is_authenticated = _webtexttool_connect('/user/authenticated', _webtexttool_get_token());
  return $is_authenticated;
}

/**
 * Set token from webtexttool.
 */
function _webtexttool_set_token($token) {
  // Make sure we can use the session.
  if (!drupal_session_started()) {
    drupal_session_initialize();
  }
  $_SESSION['webttextool_token'] = $token;
}

/**
 * Get token from webtexttool.
 */
function _webtexttool_get_token() {
  // Make sure we can use the session.
  if (!drupal_session_started()) {
    drupal_session_initialize();
  }
  if (isset($_SESSION['webttextool_token']) && $_SESSION['webttextool_token'] != '') {
    return $_SESSION['webttextool_token'];
  }
  else {
    $result = _webtexttool_login();
    if ($result && isset($result['access_token']) && $result['access_token'] != '') {
      return $result['access_token'];
    }
  }
  return FALSE;
}

/**
 * Implements getkeywordsources from api.
 */
function webtexttool_getkeywordsources() {
  if ($cache = cache_get('webtexttool_getkeywordsources')) {
    $keywords = $cache->data;
  }
  else {
    $keywords = array();
    // Get the current account.
    $keywords_languages = _webtexttool_connect('/keywords/sources', _webtexttool_get_token());
    if ($keywords_languages) {
      foreach ($keywords_languages as $keyword) {
        $keywords[$keyword['Value']] = $keyword['Name'];
      }
      cache_set('webtexttool_getkeywordsources', $keywords, 'cache');
    }
  }

  return $keywords;
}

/**
 * Implements searchkeyword from api.
 */
function webtexttool_searchkeyword($keyword, $database = 'us', $language = 'en') {
  $response = _webtexttool_connect('/keywords/' . drupal_encode_path($keyword). '/'. $database . '/' . $language, _webtexttool_get_token());
  return $response;
}

/**
 * Implements searchkeyword from api.
 */
function webtexttool_analyse($html, $keywords, $language = 'en') {
  $parameters = array(
    'content' => $html,
    'keywords' => $keywords,
    'languageCode' => variable_get('webtexttool_language', 'en'),
  );
  $response = _webtexttool_connect('/page/suggestions', _webtexttool_get_token(), $parameters, 'POST');

  // Adding a complete percentage of all the rules per suggestion.
  if (isset($response['Suggestions']) && is_array($response['Suggestions'])) {
    foreach ($response['Suggestions'] as $key => $suggestion) {
      $response['Suggestions'][$key]['complete'] = 0;
      $total_rules = 0;
      $total_checked = 0;
      if (isset($suggestion['Rules']) && is_array($suggestion['Rules'])) {
        foreach ($suggestion['Rules'] as $rule) {
          $total_rules++;

          if ($rule['Checked']) {
            $total_checked++;
          }
        }
      }

      $percentage = round(($total_checked / $total_rules) * 100);
      $response['Suggestions'][$key]['complete'] = $percentage;
      // Add a percentage class for the interface.
      if ($percentage == 0) {
        $response['Suggestions'][$key]['complete-class'] = 'score-none';
      }
      elseif ($percentage < 37) {
        $response['Suggestions'][$key]['complete-class'] = 'score-low';
      }
      elseif ($percentage > 37 && $percentage < 67) {
        $response['Suggestions'][$key]['complete-class'] = 'score-moderate';
      }
      elseif ($percentage > 67 && $percentage < 100) {
        $response['Suggestions'][$key]['complete-class'] = 'score-high';
      }
      elseif ($percentage == 100) {
        $response['Suggestions'][$key]['complete-class'] = 'score-complete';
      }
    }
  }
  if ($response) {
    foreach ($response as $key => $value) {
      // Change all keys to lower.
      $temp_key = strtolower($key);
      unset($response[$key]);
      $response[$temp_key] = $value;
    }
  }
  return $response;
}

/**
 * Helper function to change overall score to text.
 */
function webtexttool_overall_to_text($score) {
  // 0 - poor, 1 = moderate, 2 - good.
  $score_array = array(
    '-1' => 'moderate',
    '0' => 'poor',
    '1' => 'moderate',
    '2' => 'good',
  );
  return $score_array[$score];
}

/**
 * Helper function to change Competition Score to text.
 */
function webtexttool_competition_score_to_text($score) {
  // -1 - N/A, 0 - very hard, 1- hard, 2 - moderate, 3 - easy, 4 - very easy.
  $score_array = array(
    '-1' => 'moderate',
    '0' => 'very hard',
    '1' => 'hard',
    '2' => 'moderate',
    '3' => 'easy',
    '4' => 'very easy',
  );
  return $score_array[$score];
}

/**
 * Helper function to change Competition Score to text.
 */
function webtexttool_voilume_score_to_text($score) {
  // VolumeScore: -1 - N/A, 0 - very low, 1 - low,
  // 2 - moderate, 3 - high, 4- very high.
  $score_array = array(
    '-1' => t('moderate'),
    '0' => t('very low'),
    '1' => t('low'),
    '2' => t('moderate'),
    '3' => t('high'),
    '4' => t('very high'),
  );
  return $score_array[$score];
}

/**
 * Helper function to add a class based on the Score.
 */
function webtexttool_score_to_class($score) {
  // -1 - N/A, 0 - very low, 1 - low, 2 - moderate, 3 - high, 4- very high.
  $class_array = array(
    '-1' => 'yellow',
    '0' => 'red',
    '1' => 'orange',
    '2' => 'yellow',
    '3' => 'light-green',
    '4' => 'green',
  );
  return $class_array[$score];
}

/**
 * Helper function to add a class based on the OverallScore.
 */
function webtexttool_overall_score_to_class($score) {
  // 0 - poor, 1 - moderate, 2 - good.
  $class_array = array(
    '-1' => 'yellow',
    '0' => 'red',
    '1' => 'yellow',
    '2' => 'green',
  );
  return $class_array[$score];
}

/**
 * Helper function to create the markup for an colord indicator.
 */
function webtexttool_color_indicator($color_class) {
  return "<span class='indicator $color_class'>&nbsp;</span>";
}

/**
 * Render acount status.
 */
function webtexttool_account_status() {
  $is_authenticated = _webtexttool_is_authenticated();
  if (!$is_authenticated) {
    _webtexttool_login();
  }
  $output = webtexttool_account();
  return $output;

}
